@@ -1,44 +1,78 @@
 use crate::{
     config::Config,
     discref::{Device, Page, PageOps, ReadPage, UpdateList},
-    error::PRes,
+    error::PERes,
     flush_checksum::{double_buffer_check, write_root_page},
     io::{read_u64, write_u64, InfallibleReadFormat},
 };
 use linked_hash_map::LinkedHashMap;
-use std::{io::Read, sync::Mutex};
+use std::{
+    io::Read,
+    sync::Mutex,
+    time::{Duration, Instant},
+};
 
 const ALLOCATOR_PAGE_EXP: u8 = 10; // 2^10
 const ALLOCATER_ROOT_PAGE_VERSION_V0: u8 = 0;
 const ALLOCATER_ROOT_PAGE_VERSION: u8 = ALLOCATER_ROOT_PAGE_VERSION_V0;
 
+struct CacheEntry {
+    page: ReadPage,
+    instant: Instant,
+}
+impl CacheEntry {
+    fn new(page: ReadPage) -> Self {
+        Self {
+            page,
+            instant: Instant::now(),
+        }
+    }
+}
+
 pub struct Cache {
-    cache: LinkedHashMap<u64, ReadPage>,
+    cache: LinkedHashMap<u64, CacheEntry>,
     size: u64,
     limit: u64,
+    age_limit: Duration,
 }
 
 impl Cache {
-    pub fn new(limit: u64) -> Cache {
+    pub fn new(limit: u64, age_limit: Duration) -> Cache {
         Cache {
             cache: LinkedHashMap::new(),
             size: 0,
             limit,
+            age_limit,
         }
     }
 
     fn get(&mut self, key: u64) -> Option<ReadPage> {
-        self.cache.get_refresh(&key).map(|val| val.clone_read())
+        self.cache.get_refresh(&key).map(|val| val.page.clone_read())
     }
 
     fn put(&mut self, key: u64, value: ReadPage) {
         self.size += 1 << value.get_size_exp();
-        if let Some(pre) = self.cache.insert(key, value) {
-            self.size -= 1 << pre.get_size_exp();
+        if let Some(pre) = self.cache.insert(key, CacheEntry::new(value)) {
+            self.size -= 1 << pre.page.get_size_exp();
         }
         while self.size > self.limit {
             if let Some(en) = self.cache.pop_front() {
-                self.size -= 1 << en.1.get_size_exp();
+                self.size -= 1 << en.1.page.get_size_exp();
+            } else {
+                break;
+            }
+        }
+
+        loop {
+            if self
+                .cache
+                .front()
+                .map(|(_, e)| e.instant.elapsed() > self.age_limit)
+                .unwrap_or(false)
+            {
+                if let Some(en) = self.cache.pop_front() {
+                    self.size -= 1 << en.1.page.get_size_exp();
+                }
             } else {
                 break;
             }
@@ -47,7 +81,7 @@ impl Cache {
 
     fn remove(&mut self, key: u64) {
         if let Some(removed) = self.cache.remove(&key) {
-            self.size -= 1 << removed.get_size_exp();
+            self.size -= 1 << removed.page.get_size_exp();
         }
     }
 }
@@ -66,13 +100,13 @@ impl FreeList {
         }
     }
 
-    fn read(page: &mut ReadPage) -> PRes<FreeList> {
+    fn read(page: &mut ReadPage) -> PERes<FreeList> {
         match page.read_u8() {
             ALLOCATER_ROOT_PAGE_VERSION_V0 => Self::read_v0(page),
             _ => panic!("unsupported format"),
         }
     }
-    fn read_v0(page: &mut ReadPage) -> PRes<FreeList> {
+    fn read_v0(page: &mut ReadPage) -> PERes<FreeList> {
         let mut buffer_0 = [0; 259];
         let mut buffer_1 = [0; 259];
         let freelist;
@@ -107,7 +141,7 @@ impl FreeList {
         buffer
     }
 
-    pub fn write_list(&mut self, page: &mut Page) -> PRes<()> {
+    pub fn write_list(&mut self, page: &mut Page) -> PERes<()> {
         let mut buffer = Self::write_free_list(&self.list);
         self.last_flush = write_root_page(page, &mut buffer, ALLOCATER_ROOT_PAGE_VERSION, self.last_flush)?;
         Ok(())
@@ -142,7 +176,7 @@ pub struct Allocator {
 }
 
 impl Allocator {
-    pub fn new(dr: Box<dyn Device>, config: &Config, page: u64) -> PRes<Allocator> {
+    pub fn new(dr: Box<dyn Device>, config: &Config, page: u64) -> PERes<Allocator> {
         let mut pg = dr.load_page(page)?;
         let mut freelist = FreeList::read(&mut pg)?;
 
@@ -159,16 +193,17 @@ impl Allocator {
         }
 
         let cache_size = config.cache_size();
+        let cache_age_limit = config.cache_age_limit();
         Ok(Allocator {
             disc: dr,
             free_list: Mutex::new(freelist),
-            cache: Mutex::new(Cache::new(cache_size)),
+            cache: Mutex::new(Cache::new(cache_size, cache_age_limit)),
             root_monitor: Default::default(),
             page,
         })
     }
 
-    pub fn init(dr: Box<dyn Device>, config: &Config) -> PRes<(u64, Allocator)> {
+    pub fn init(dr: Box<dyn Device>, config: &Config) -> PERes<(u64, Allocator)> {
         let mut page = dr.create_page(ALLOCATOR_PAGE_EXP)?;
         let mut list = FreeList::new([0; 32], 0);
         list.write_list(&mut page)?;
@@ -177,7 +212,7 @@ impl Allocator {
         Ok((allocate_page, Allocator::new(dr, config, allocate_page)?))
     }
 
-    pub fn load_page_not_free(&self, page: u64) -> PRes<Option<ReadPage>> {
+    pub fn load_page_not_free(&self, page: u64) -> PERes<Option<ReadPage>> {
         {
             let mut cache = self.cache.lock()?;
             if let Some(pg) = cache.get(page) {
@@ -201,19 +236,19 @@ impl Allocator {
         }
     }
 
-    pub fn load_page(&self, page: u64) -> PRes<ReadPage> {
+    pub fn load_page(&self, page: u64) -> PERes<ReadPage> {
         let load = self.read_page_int(page)?;
         debug_assert!(!load.is_free()?, "page {} should not be marked as free", page);
         Ok(load)
     }
 
-    pub fn write_page(&self, page: u64) -> PRes<Page> {
+    pub fn write_page(&self, page: u64) -> PERes<Page> {
         let load = self.write_page_int(page)?;
         debug_assert!(!load.is_free()?, "page {} should not be marked as free", page);
         Ok(load)
     }
 
-    fn read_page_int(&self, page: u64) -> PRes<ReadPage> {
+    fn read_page_int(&self, page: u64) -> PERes<ReadPage> {
         {
             let mut cache = self.cache.lock()?;
             if let Some(pg) = cache.get(page) {
@@ -228,7 +263,7 @@ impl Allocator {
         Ok(load)
     }
 
-    fn write_page_int(&self, page: u64) -> PRes<Page> {
+    fn write_page_int(&self, page: u64) -> PERes<Page> {
         let cache_result;
         {
             let mut cache = self.cache.lock()?;
@@ -245,7 +280,7 @@ impl Allocator {
         Ok(load.clone_write())
     }
 
-    pub fn allocate(&self, exp: u8) -> PRes<Page> {
+    pub fn allocate(&self, exp: u8) -> PERes<Page> {
         {
             let mut fl = self.free_list.lock()?;
             let page = fl.get(exp);
@@ -261,7 +296,7 @@ impl Allocator {
         Ok(page)
     }
 
-    pub fn flush_page(&self, page: Page) -> PRes<()> {
+    pub fn flush_page(&self, page: Page) -> PERes<()> {
         self.disc.flush_page(&page)?;
         {
             let mut cache = self.cache.lock()?;
@@ -270,7 +305,7 @@ impl Allocator {
         Ok(())
     }
 
-    pub fn remove_from_free(&self, page: u64, exp: u8) -> PRes<()> {
+    pub fn remove_from_free(&self, page: u64, exp: u8) -> PERes<()> {
         let mut fl = self.free_list.lock()?;
         let mut pg = self.write_page_int(page)?;
         if pg.is_free()? {
@@ -310,7 +345,7 @@ impl Allocator {
     }
 
     /// Recover free do not not have debug asserts for already freed pages
-    pub fn recover_free(&self, page: u64) -> PRes<()> {
+    pub fn recover_free(&self, page: u64) -> PERes<()> {
         if let Ok(p) = self.disc.load_page(page) {
             if !p.is_free()? {
                 self.free(page)?;
@@ -318,14 +353,14 @@ impl Allocator {
         }
         Ok(())
     }
-    pub fn trim_free_at_end(&self) -> PRes<()> {
+    pub fn trim_free_at_end(&self) -> PERes<()> {
         let mut fl = self.free_list.lock()?;
         let list: &mut FreeList = &mut fl;
         self.disc.trim_end_pages(list)?;
         Ok(())
     }
 
-    pub fn free(&self, page: u64) -> PRes<()> {
+    pub fn free(&self, page: u64) -> PERes<()> {
         self.cache.lock()?.remove(page);
         let mut fl = self.free_list.lock()?;
         let list: &mut FreeList = &mut fl;
@@ -333,7 +368,7 @@ impl Allocator {
         Ok(())
     }
 
-    pub fn flush_free_list(&self) -> PRes<()> {
+    pub fn flush_free_list(&self) -> PERes<()> {
         let mut lock = self.free_list.lock()?;
         if lock.to_flush {
             let mut monitor = self.root_monitor.lock()?;
@@ -350,7 +385,7 @@ impl Allocator {
         Ok(())
     }
 
-    pub fn write_address_root(&self, root: Page, buffer: &mut [u8], version: u8, last_flush: u8) -> PRes<u8> {
+    pub fn write_address_root(&self, root: Page, buffer: &mut [u8], version: u8, last_flush: u8) -> PERes<u8> {
         let mut monitor = self.root_monitor.lock()?;
         if monitor.address_dirty {
             self.disc().sync()?;
@@ -359,7 +394,7 @@ impl Allocator {
         monitor.address_dirty = true;
         Ok(order)
     }
-    pub fn write_juornal_root(&self, root: Page, buffer: &mut [u8], version: u8, last_flush: u8) -> PRes<u8> {
+    pub fn write_juornal_root(&self, root: Page, buffer: &mut [u8], version: u8, last_flush: u8) -> PERes<u8> {
         let mut monitor = self.root_monitor.lock()?;
         if monitor.journal_dirty {
             self.disc().sync()?;
@@ -369,13 +404,13 @@ impl Allocator {
         Ok(order)
     }
 
-    pub fn write_root_page(&self, mut root: Page, buffer: &mut [u8], version: u8, last_flush: u8) -> PRes<u8> {
+    pub fn write_root_page(&self, mut root: Page, buffer: &mut [u8], version: u8, last_flush: u8) -> PERes<u8> {
         let order = write_root_page(&mut root, buffer, version, last_flush)?;
         self.flush_page(root)?;
         Ok(order)
     }
 
-    pub fn flush_root_page(&self, page: Page) -> PRes<()> {
+    pub fn flush_root_page(&self, page: Page) -> PERes<()> {
         self.flush_page(page)
     }
 
@@ -383,7 +418,7 @@ impl Allocator {
         &*self.disc
     }
 
-    pub fn disc_sync(&self) -> PRes<()> {
+    pub fn disc_sync(&self) -> PERes<()> {
         let mut monitor = self.root_monitor.lock()?;
         self.disc.sync()?;
         monitor.free_list_dirty = false;
@@ -398,13 +433,13 @@ impl Allocator {
 }
 
 impl UpdateList for FreeList {
-    fn update(&mut self, size: u8, page: u64) -> PRes<u64> {
+    fn update(&mut self, size: u8, page: u64) -> PERes<u64> {
         let old = self.set(size, page);
         debug_assert!(old != page || old == 0, "freeing: {} already free: {} ", page, old);
         Ok(old)
     }
 
-    fn remove(&mut self, size: u8, page: u64, next: u64) -> PRes<()> {
+    fn remove(&mut self, size: u8, page: u64, next: u64) -> PERes<()> {
         let old = self.set(size, next);
         debug_assert!(
             old == page || old == 0,
@@ -423,7 +458,7 @@ mod tests {
         config::Config,
         discref::{DiscRef, PageOps, ReadPage},
     };
-    use std::sync::Arc;
+    use std::{sync::Arc, time::Duration};
     use tempfile::Builder;
 
     #[test]
@@ -478,7 +513,6 @@ mod tests {
         let second = allocator.allocate(10).unwrap().get_index();
         let third = allocator.allocate(10).unwrap().get_index();
         let _forth_to_avoid_trim = allocator.allocate(11).unwrap();
-        println!("{} {} {}", first, second, third);
         allocator.free(first).unwrap();
         allocator.free(second).unwrap();
         allocator.free(third).unwrap();
@@ -491,7 +525,7 @@ mod tests {
 
     #[test]
     fn test_cache_limit_evict() {
-        let mut cache = Cache::new(1050 as u64);
+        let mut cache = Cache::new(1050 as u64, Duration::from_secs(3600));
         cache.put(10, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
         cache.put(20, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
         cache.put(30, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
@@ -516,7 +550,7 @@ mod tests {
 
     #[test]
     fn test_cache_limit_refresh_evict() {
-        let mut cache = Cache::new(1050 as u64);
+        let mut cache = Cache::new(1050 as u64, Duration::from_secs(3600));
         cache.put(10, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
         cache.put(20, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
         let ten = 10 as u64;
@@ -559,7 +593,7 @@ mod tests {
 
     #[test]
     fn test_cache_limit_remove() {
-        let mut cache = Cache::new(1050 as u64);
+        let mut cache = Cache::new(1050 as u64, Duration::from_secs(3600));
         cache.put(10, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
         cache.put(20, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
         cache.put(30, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
@@ -572,7 +606,7 @@ mod tests {
 
     #[test]
     fn test_cache_replace_limit_stay() {
-        let mut cache = Cache::new(100000 as u64);
+        let mut cache = Cache::new(100000 as u64, Duration::from_secs(3600));
         cache.put(10, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
         cache.put(20, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
         cache.put(30, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
@@ -582,4 +616,15 @@ mod tests {
         cache.put(20, ReadPage::new(Arc::new(Vec::new()), 0, 10, 10));
         assert_eq!(cache.size, pre - (1 << 9) + (1 << 10));
     }
+
+    #[test]
+    fn test_cache_time_clean() {
+        let mut cache = Cache::new(100000 as u64, Duration::from_millis(500));
+        cache.put(10, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
+        cache.put(20, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
+        cache.put(30, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
+        std::thread::sleep(Duration::from_millis(600));
+        cache.put(20, ReadPage::new(Arc::new(Vec::new()), 0, 10, 9));
+        assert_eq!(cache.size, 1 << 9);
+    }
 }
