@@ -1,11 +1,10 @@
 use crate::{
-    error::PersyError,
+    error::{IndexChangeError, PIRes},
+    id::RecRef,
     index::{
         config::{IndexOrd, IndexType, ValueMode},
         tree::PosRef,
     },
-    persy::RecRef,
-    PRes,
 };
 use std::{
     cmp::Ordering,
@@ -17,27 +16,27 @@ use std::{
 pub type NodeRef = RecRef;
 #[derive(Clone)]
 pub enum Node<K, V> {
-    NODE(Nodes<K>),
-    LEAF(Leaf<K, V>),
+    Node(Nodes<K>),
+    Leaf(Leaf<K, V>),
 }
 
 impl<K, V> Node<K, V> {
     pub fn get_prev(&self) -> &Option<K> {
         match self {
-            Node::NODE(n) => &n.prev,
-            Node::LEAF(l) => &l.prev,
+            Node::Node(n) => &n.prev,
+            Node::Leaf(l) => &l.prev,
         }
     }
     pub fn get_next(&self) -> &Option<K> {
         match self {
-            Node::NODE(n) => &n.next,
-            Node::LEAF(l) => &l.next,
+            Node::Node(n) => &n.next,
+            Node::Leaf(l) => &l.next,
         }
     }
     pub fn len(&self) -> usize {
         match self {
-            Node::NODE(n) => n.len(),
-            Node::LEAF(l) => l.len(),
+            Node::Node(n) => n.len(),
+            Node::Leaf(l) => l.len(),
         }
     }
 }
@@ -45,19 +44,19 @@ impl<K, V> Node<K, V> {
 impl<K: IndexOrd + Clone, V> Node<K, V> {
     pub fn merge_right(&mut self, k: K, node: &mut Node<K, V>) {
         match self {
-            Node::NODE(n) => match node {
-                Node::NODE(n1) => {
+            Node::Node(n) => match node {
+                Node::Node(n1) => {
                     n.merge_right(k, n1);
                 }
-                Node::LEAF(_) => {
+                Node::Leaf(_) => {
                     panic!("impossible merge a leaf to node");
                 }
             },
-            Node::LEAF(l) => match node {
-                Node::NODE(_) => {
+            Node::Leaf(l) => match node {
+                Node::Node(_) => {
                     panic!("impossible merge a node to leaf");
                 }
-                Node::LEAF(l1) => {
+                Node::Leaf(l1) => {
                     l.merge_right(l1);
                 }
             },
@@ -66,15 +65,15 @@ impl<K: IndexOrd + Clone, V> Node<K, V> {
 
     pub fn split(&mut self, top_limit: usize) -> Vec<(K, Node<K, V>)> {
         match self {
-            Node::NODE(n) => n.split(top_limit).into_iter().map(|x| (x.0, Node::NODE(x.1))).collect(),
-            Node::LEAF(l) => l.split(top_limit).into_iter().map(|x| (x.0, Node::LEAF(x.1))).collect(),
+            Node::Node(n) => n.split(top_limit).into_iter().map(|x| (x.0, Node::Node(x.1))).collect(),
+            Node::Leaf(l) => l.split(top_limit).into_iter().map(|x| (x.0, Node::Leaf(x.1))).collect(),
         }
     }
 
     pub fn check_range(&self, k: &K) -> bool {
         match self {
-            Node::NODE(n) => n.check_range(k),
-            Node::LEAF(l) => l.check_range(k),
+            Node::Node(n) => n.check_range(k),
+            Node::Leaf(l) => l.check_range(k),
         }
     }
 }
@@ -170,7 +169,7 @@ impl<K: IndexOrd + Clone> Nodes<K> {
     pub fn insert_after(&mut self, pos: usize, values: &mut Vec<(K, NodeRef)>) {
         values.reverse();
         for (key, node) in values.iter() {
-            self.add(pos, &key, node.clone());
+            self.add(pos, key, node.clone());
         }
     }
 
@@ -221,8 +220,8 @@ impl<K: IndexOrd + Clone> Nodes<K> {
 
     #[allow(dead_code)]
     pub fn merge_left(&mut self, owner: K, nodes: &mut Nodes<K>) {
-        let mut keys = std::mem::replace(&mut nodes.keys, Vec::new());
-        let mut pointers = std::mem::replace(&mut nodes.pointers, Vec::new());
+        let mut keys = std::mem::take(&mut nodes.keys);
+        let mut pointers = std::mem::take(&mut nodes.pointers);
         keys.push(owner);
         keys.append(&mut self.keys);
         pointers.append(&mut self.pointers);
@@ -256,9 +255,9 @@ impl<K: IndexOrd + Clone> Nodes<K> {
 #[derive(Clone, PartialEq, Debug)]
 pub enum Value<V> {
     /// A cluster of values
-    CLUSTER(Vec<V>),
+    Cluster(Vec<V>),
     /// A single value entry
-    SINGLE(V),
+    Single(V),
 }
 
 impl<V> IntoIterator for Value<V> {
@@ -267,8 +266,8 @@ impl<V> IntoIterator for Value<V> {
 
     fn into_iter(self) -> IntoIter<V> {
         match self {
-            Value::SINGLE(v) => vec![v].into_iter(),
-            Value::CLUSTER(v) => v.into_iter(),
+            Value::Single(v) => vec![v].into_iter(),
+            Value::Cluster(v) => v.into_iter(),
         }
     }
 }
@@ -309,7 +308,7 @@ impl<K, V> Leaf<K, V> {
 
     #[allow(dead_code)]
     pub fn merge_left(&mut self, leaf: &mut Leaf<K, V>) {
-        let mut entries = std::mem::replace(&mut leaf.entries, Vec::new());
+        let mut entries = std::mem::take(&mut leaf.entries);
         entries.append(&mut self.entries);
         self.entries = entries;
     }
@@ -385,7 +384,7 @@ impl<K: IndexOrd + Clone, V: Clone> Leaf<K, V> {
             pos,
             LeafEntry {
                 key: k.clone(),
-                value: Value::SINGLE(v.clone()),
+                value: Value::Single(v.clone()),
             },
         );
     }
@@ -422,35 +421,38 @@ impl<K: IndexOrd + Clone, V: Clone> Leaf<K, V> {
 }
 
 impl<K: IndexType, V: IndexOrd + Clone> Leaf<K, V> {
-    pub fn insert_or_update(&mut self, k: &K, v: &V, value_mode: ValueMode, index_name: &str) -> PRes<()> {
+    pub fn insert_or_update(&mut self, k: &K, v: &V, value_mode: ValueMode, index_name: &str) -> PIRes<()> {
         match self.entries.binary_search_by(|n| compare(&n.key, k)) {
             Ok(index) => {
                 let entry = &mut self.entries[index];
                 match value_mode {
-                    ValueMode::REPLACE => {
-                        entry.value = Value::SINGLE(v.clone());
+                    ValueMode::Replace => {
+                        entry.value = Value::Single(v.clone());
                     }
-                    ValueMode::EXCLUSIVE => match entry.value {
-                        Value::SINGLE(ref ev) => {
+                    ValueMode::Exclusive => match entry.value {
+                        Value::Single(ref ev) => {
                             if compare(ev, v) != Ordering::Equal {
-                                return Err(PersyError::IndexDuplicateKey(index_name.to_string(), format!("{}", k)));
+                                return Err(IndexChangeError::IndexDuplicateKey(
+                                    index_name.to_string(),
+                                    format!("{}", k),
+                                ));
                             }
                         }
                         _ => unreachable!("Exclusive leafs never have cluster values"),
                     },
-                    ValueMode::CLUSTER => {
+                    ValueMode::Cluster => {
                         let mut new_value = None;
                         match entry.value {
-                            Value::SINGLE(ref ev) => match compare(ev, v) {
+                            Value::Single(ref ev) => match compare(ev, v) {
                                 Ordering::Equal => {}
                                 Ordering::Less => {
-                                    new_value = Some(Value::CLUSTER(vec![ev.clone(), v.clone()]));
+                                    new_value = Some(Value::Cluster(vec![ev.clone(), v.clone()]));
                                 }
                                 Ordering::Greater => {
-                                    new_value = Some(Value::CLUSTER(vec![v.clone(), ev.clone()]));
+                                    new_value = Some(Value::Cluster(vec![v.clone(), ev.clone()]));
                                 }
                             },
-                            Value::CLUSTER(ref mut cl) => {
+                            Value::Cluster(ref mut cl) => {
                                 if let Err(index) = cl.binary_search_by(|x| compare(x, v)) {
                                     cl.insert(index, v.clone());
                                 }
@@ -476,7 +478,7 @@ impl<K: IndexType, V: IndexOrd + Clone> Leaf<K, V> {
                         let mut new_value = None;
                         let entry = &mut self.entries[index];
                         let remove_entry = match &mut entry.value {
-                            Value::SINGLE(val) => {
+                            Value::Single(val) => {
                                 if compare(val, rv) == Ordering::Equal {
                                     removed = true;
                                     true
@@ -484,13 +486,13 @@ impl<K: IndexType, V: IndexOrd + Clone> Leaf<K, V> {
                                     false
                                 }
                             }
-                            Value::CLUSTER(ref mut cl) => {
+                            Value::Cluster(ref mut cl) => {
                                 if let Ok(index) = cl.binary_search_by(|x| compare(x, rv)) {
                                     removed = true;
                                     cl.remove(index);
                                 }
                                 if cl.len() == 1 {
-                                    new_value = Some(Value::SINGLE(cl.pop().unwrap()));
+                                    new_value = Some(Value::Single(cl.pop().unwrap()));
                                     false
                                 } else {
                                     cl.is_empty()
@@ -566,11 +568,11 @@ mod tests {
     fn single_leaf_insert_test() {
         let mut leaf = Leaf::new();
         for n in 0..50 {
-            leaf.insert_or_update(&n, &n, ValueMode::REPLACE, "aa")
+            leaf.insert_or_update(&n, &n, ValueMode::Replace, "aa")
                 .expect("insert is ok");
         }
         let res = leaf.find(&10);
-        assert_eq!(Ok((10, Value::SINGLE(10))), res);
+        assert_eq!(Ok((10, Value::Single(10))), res);
 
         let res = leaf.find(&60);
         assert_eq!(Err(50), res);
@@ -579,54 +581,54 @@ mod tests {
     #[test]
     fn single_leaf_cluster_insert_test() {
         let mut leaf = Leaf::new();
-        leaf.insert_or_update(&10, &1, ValueMode::CLUSTER, "aa")
+        leaf.insert_or_update(&10, &1, ValueMode::Cluster, "aa")
             .expect("insert is ok");
-        leaf.insert_or_update(&10, &2, ValueMode::CLUSTER, "aa")
+        leaf.insert_or_update(&10, &2, ValueMode::Cluster, "aa")
             .expect("insert is ok");
         let res = leaf.find(&10);
-        assert_eq!(Ok((10, Value::CLUSTER(vec![1, 2]))), res);
+        assert_eq!(Ok((10, Value::Cluster(vec![1, 2]))), res);
     }
 
     #[test]
     fn leaf_cluster_remove_test() {
         let mut leaf = Leaf::new();
-        leaf.insert_or_update(&10, &1, ValueMode::CLUSTER, "aa")
+        leaf.insert_or_update(&10, &1, ValueMode::Cluster, "aa")
             .expect("insert is ok");
-        leaf.insert_or_update(&10, &2, ValueMode::CLUSTER, "aa")
+        leaf.insert_or_update(&10, &2, ValueMode::Cluster, "aa")
             .expect("insert is ok");
         assert!(leaf.remove(&10, &Some(2)));
         let res = leaf.find(&10);
-        assert_eq!(Ok((10, Value::SINGLE(1))), res);
+        assert_eq!(Ok((10, Value::Single(1))), res);
     }
 
     #[test]
     fn leaf_cluster_remove_not_exist_value_test() {
         let mut leaf = Leaf::new();
-        leaf.insert_or_update(&10, &1, ValueMode::CLUSTER, "aa")
+        leaf.insert_or_update(&10, &1, ValueMode::Cluster, "aa")
             .expect("insert is ok");
-        leaf.insert_or_update(&10, &2, ValueMode::CLUSTER, "aa")
+        leaf.insert_or_update(&10, &2, ValueMode::Cluster, "aa")
             .expect("insert is ok");
         assert!(!leaf.remove(&10, &Some(10)));
         let res = leaf.find(&10);
-        assert_eq!(Ok((10, Value::CLUSTER(vec![1, 2]))), res);
+        assert_eq!(Ok((10, Value::Cluster(vec![1, 2]))), res);
     }
 
     #[test]
     fn leaf_single_delete_not_exist_value_test() {
         let mut leaf = Leaf::new();
-        leaf.insert_or_update(&10, &1, ValueMode::EXCLUSIVE, "aa")
+        leaf.insert_or_update(&10, &1, ValueMode::Exclusive, "aa")
             .expect("insert is ok");
         assert!(!leaf.remove(&10, &Some(10)));
         let res = leaf.find(&10);
-        assert_eq!(Ok((10, Value::SINGLE(1))), res);
+        assert_eq!(Ok((10, Value::Single(1))), res);
     }
 
     #[test]
     fn leaf_duplicate_key_test() {
         let mut leaf = Leaf::new();
-        leaf.insert_or_update(&10, &1, ValueMode::EXCLUSIVE, "aa")
+        leaf.insert_or_update(&10, &1, ValueMode::Exclusive, "aa")
             .expect("insert is ok");
-        let res = leaf.insert_or_update(&10, &2, ValueMode::EXCLUSIVE, "aa");
+        let res = leaf.insert_or_update(&10, &2, ValueMode::Exclusive, "aa");
         assert!(res.is_err());
     }
 
@@ -635,7 +637,7 @@ mod tests {
         let mut leaf = Leaf::new();
 
         for n in 0..103 {
-            leaf.insert_or_update(&n, &n, ValueMode::REPLACE, "aa")
+            leaf.insert_or_update(&n, &n, ValueMode::Replace, "aa")
                 .expect("insert is ok");
         }
 
@@ -677,7 +679,7 @@ mod tests {
     fn test_remove_from_leaf() {
         let mut leaf = Leaf::new();
         for n in 0..50 {
-            leaf.insert_or_update(&n, &n, ValueMode::REPLACE, "aa")
+            leaf.insert_or_update(&n, &n, ValueMode::Replace, "aa")
                 .expect("insert is ok");
         }
         assert!(leaf.remove(&10, &Some(10)));
@@ -716,38 +718,38 @@ mod tests {
         let mut leaf = Leaf::new();
         let mut leaf2 = Leaf::new();
         for n in 0..20 {
-            leaf.insert_or_update(&n, &n, ValueMode::REPLACE, "aa")
+            leaf.insert_or_update(&n, &n, ValueMode::Replace, "aa")
                 .expect("insert is ok");
         }
 
         for n in 20..40 {
             leaf2
-                .insert_or_update(&n, &n, ValueMode::REPLACE, "aa")
+                .insert_or_update(&n, &n, ValueMode::Replace, "aa")
                 .expect("insert is ok");
         }
         leaf.merge_right(&mut leaf2);
         assert_eq!(leaf.len(), 40);
         assert_eq!(leaf2.len(), 0);
         let res = leaf.find(&35);
-        assert_eq!(res, Ok((35, Value::SINGLE(35))));
+        assert_eq!(res, Ok((35, Value::Single(35))));
 
         let mut leaf = Leaf::new();
         let mut leaf2 = Leaf::new();
         for n in 20..40 {
-            leaf.insert_or_update(&n, &n, ValueMode::REPLACE, "aa")
+            leaf.insert_or_update(&n, &n, ValueMode::Replace, "aa")
                 .expect("insert is ok");
         }
 
         for n in 0..20 {
             leaf2
-                .insert_or_update(&n, &n, ValueMode::REPLACE, "aa")
+                .insert_or_update(&n, &n, ValueMode::Replace, "aa")
                 .expect("insert is ok");
         }
         leaf.merge_left(&mut leaf2);
         assert_eq!(leaf.len(), 40);
         assert_eq!(leaf2.len(), 0);
         let res = leaf.find(&35);
-        assert_eq!(res, Ok((35, Value::SINGLE(35))));
+        assert_eq!(res, Ok((35, Value::Single(35))));
     }
 
     #[test]
