@@ -296,6 +296,8 @@ impl PersyImpl {
         for (_, (_, tx, _)) in recover.transactions.iter_mut() {
             tx.recover_rollback(self)?;
         }
+        self.address.recompute_last_pages()?;
+        self.address.flush_segments()?;
         allocator.disc_sync()?;
         self.journal.clear_in_queque()?;
         allocator.trim_free_at_end()?;
